@@ -115,7 +115,9 @@ ges_video_source_create_filters (GESVideoSource * self, GPtrArray * elements,
   GESTrackElement *trksrc = GES_TRACK_ELEMENT (self);
   GstElement *positioner, *videoflip, *capsfilter, *videorate;
   const gchar *positioner_props[]
-  = { "alpha", "posx", "posy", "width", "height", "operator", NULL };
+      = { "alpha", "posx", "fposx", "posy", "fposy", "width", "fwidth",
+    "height", "fheight", "operator", NULL
+  };
   const gchar *videoflip_props[] = { "video-direction", NULL };
   gchar *ename = NULL;
 
@@ -205,6 +207,7 @@ ges_video_source_create_element (GESTrackElement * trksrc)
   g_assert (vsource_class->ABI.abi.create_filters);
   if (!vsource_class->ABI.abi.create_filters (self, elements, needs_converters)) {
     g_ptr_array_free (elements, TRUE);
+    gst_object_unref (sub_element);
 
     return NULL;
   }
